<?php
namespace App\Controller;

use App\Templates;
use App\Exception;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

use SimpleXMLElement;

class WebService {

    private $util;
    private $integrationService;
    
    private $basePath;
    private $ordersPath;        
    private $processedPath;     
    private $errPath;
    
    private $mailList;
    private $serverName;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util,
                                 \App\Service\IntegrationService $integrationService) {

        $this->connection         = $connection;
        $this->util               = $util;
        $this->integrationService = $integrationService;
    }
    
    public function BalanceInquiry(Request $request, Response $response) {
        try {
            $headers = $request->getServerParams();
            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $authToken = $config['authToken'];
            
            $req = $request->getParsedBody();
            
            $json = json_encode($req);
            $array = json_decode($json, TRUE);
            
            $array = $this->formatArray($array);
            
            $inputXml = $this->integrationService->buildXmlString($array);
            
            $fileName = date("c") . ".txt";

	        $content  = "[REQUEST] - " . date("c") . "\n" . $inputXml . "\n\n";
	        
	        $this->integrationService->saveRequestLog($fileName, $content, "BalanceInquiry");
            
            if(!empty($headers['HTTP_AUTHORIZATION'])){
                
                $token = substr($headers['HTTP_AUTHORIZATION'], 7);
                
                if($token === $authToken){
                    $authToken = $this->getCurrentAuthToken();
                    
                    $requestHeader = [
                        "Content-Type: application/json",
                        "Authorization: Bearer $authToken"
                    ];
                    
                    $requestBody = $array;
                    
                    $requestResponse = $this->sendRequest("/BalanceInquiry", 'POST', $requestHeader, $requestBody);
                    $error = false;
                    
                    if(empty($requestResponse)){
                        $responseXml = $this->integrationService->buildXmlString([], 'Não foi possivel comunicar com a api Madero Core Bancario.', true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na consulta de saldo: Não foi possivel comunicar com a api Madero Core Bancario\n");
                    } 
                    
                    else if(!empty($requestResponse['isSuccess']) && $requestResponse['isSuccess'] == true){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, '', false);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Consulta de saldo realizada com sucesso.\n");
                    }
                    
                    else if(!empty($requestResponse['errors']['message'])){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, $requestResponse['errors']['message'], true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na consulta de saldo: " . $requestResponse['errors']['message'] . "\n");
                    }
                    
                    else{
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, "Ocorreu um erro inesperado.", true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na consulta de saldo: Erro inesperado.\n");
                    }
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "BalanceInquiry");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                } else{
                    $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao invalido', true);
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao invalido.\n");
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "BalanceInquiry");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                }
            } else{
                $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao nao enviado', true);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao nao enviado.\n");
                
                $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                $this->integrationService->saveRequestLog($fileName, $content, "BalanceInquiry");
                
                return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
            }
            
        } catch (\Exception $e) {
            $responseXml = $this->integrationService->buildXmlString([], 'Ocorreu um erro inesperado', true);
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro inesperado: " . $e->getMessage() . "\n");
            
            $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
            $this->integrationService->saveRequestLog($fileName, $content, "BalanceInquiry");
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
        }
    }
    
    public function BalanceInquiryResults(Request $request, Response $response) {
        try {
            $headers = $request->getServerParams();
            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $authToken = $config['authToken'];
            
            $req = $request->getParsedBody();
            
            $json = json_encode($req);
            $array = json_decode($json, TRUE);
            
            $array = $this->formatArray($array);
            
            $inputXml = $this->integrationService->buildXmlString($array);
            
            $fileName = date("c") . ".txt";

	        $content  = "[REQUEST] - " . date("c") . "\n" . $inputXml . "\n\n";
	        
	        $this->integrationService->saveRequestLog($fileName, $content, "BalanceInquiryResults");
            
            if(!empty($headers['HTTP_AUTHORIZATION'])){
                
                $token = substr($headers['HTTP_AUTHORIZATION'], 7);
                
                if($token === $authToken){
                    $authToken = $this->getCurrentAuthToken();
                    
                    $requestHeader = [
                        "Content-Type: application/json",
                        "Authorization: Bearer $authToken"
                    ];
                    
                    $requestResponse = $this->sendRequest("/BalanceInquiry/" . $array['requestId'], 'GET', $requestHeader, []);
                    $error = false;
                    
                    if(empty($requestResponse)){
                        $responseXml = $this->integrationService->buildXmlString([], 'Não foi possivel comunicar com a api Madero Core Bancario.', true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na busca de resultados da consulta de saldo: Não foi possivel comunicar com a api Madero Core Bancario\n");
                    } 
                    
                    else if(!empty($requestResponse['isSuccess']) && $requestResponse['isSuccess'] == true){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, '', false);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Busca de resultados da consulta de saldo realizada com sucesso.\n");
                    }
                    
                    else if(!empty($requestResponse['errors']['message'])){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, $requestResponse['errors']['message'], true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na busca de resultados da consulta de saldo: " . $response['errors']['message'] . "\n");
                    }
                    
                    else{
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, "Ocorreu um erro inesperado.", true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na busca de resultados da consulta de saldo: Erro inesperado.\n");
                    }
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "BalanceInquiryResults");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                } else{
                    $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao invalido', true);
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao invalido.\n");
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "BalanceInquiryResults");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                }
            } else{
                $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao nao enviado', true);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao nao enviado.\n");
                
                $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                $this->integrationService->saveRequestLog($fileName, $content, "BalanceInquiryResults");
                
                return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
            }
            
        } catch (\Exception $e) {
            $responseXml = $this->integrationService->buildXmlString([], 'Ocorreu um erro inesperado', true);
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro inesperado: " . $e->getMessage() . "\n");
            
            $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
            $this->integrationService->saveRequestLog($fileName, $content, "BalanceInquiryResults");
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
        }
    }
    
    public function Redemption(Request $request, Response $response) {
        try {
            $headers = $request->getServerParams();
            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $authToken = $config['authToken'];
            
            $req = $request->getParsedBody();
            
            $json = json_encode($req);
            $array = json_decode($json, TRUE);
            
            $array = $this->formatArray($array);
            
            $inputXml = $this->integrationService->buildXmlString($array);
            
            $fileName = date("c") . ".txt";

	        $content  = "[REQUEST] - " . date("c") . "\n" . $inputXml . "\n\n";
	        
	        $this->integrationService->saveRequestLog($fileName, $content, "Redemption");
            
            if(!empty($headers['HTTP_AUTHORIZATION'])){
                
                $token = substr($headers['HTTP_AUTHORIZATION'], 7);
                
                if($token === $authToken){
                    $authToken = $this->getCurrentAuthToken();
                    
                    $requestHeader = [
                        "Content-Type: application/json",
                        "Authorization: Bearer $authToken"
                    ];
                    
                    $requestBody = $array;
                    
                    $requestResponse = $this->sendRequest("/Redemption", 'POST', $requestHeader, $requestBody);
                    $error = false;
                    
                    if(empty($requestResponse)){
                        $responseXml = $this->integrationService->buildXmlString([], 'Não foi possivel comunicar com a api Madero Core Bancario.', true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro no resgate de saldo: Não foi possivel comunicar com a api Madero Core Bancario\n");
                    } 
                    
                    else if(!empty($requestResponse['isSuccess']) && $requestResponse['isSuccess'] == true){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, '', false);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Resgate de saldo realizado com sucesso.\n");
                    }
                    
                    else if(!empty($requestResponse['errors']['message'])){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, $requestResponse['errors']['message'], true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro no resgate de saldo: " . $requestResponse['errors']['message'] . "\n");
                    }
                    
                    else{
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, "Ocorreu um erro inesperado.", true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro no resgate de saldo: Erro inesperado.\n");
                    }
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "Redemption");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                } else{
                    $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao invalido', true);
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao invalido.\n");
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "Redemption");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                }
            } else{
                $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao nao enviado', true);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao nao enviado.\n");
                
                $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                $this->integrationService->saveRequestLog($fileName, $content, "Redemption");
                
                return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
            }
            
        } catch (\Exception $e) {
            $responseXml = $this->integrationService->buildXmlString([], 'Ocorreu um erro inesperado', true);
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro inesperado: " . $e->getMessage() . "\n");
            
            $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
            $this->integrationService->saveRequestLog($fileName, $content, "Redemption");
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
        }
    }
    
    public function RedemptionResults(Request $request, Response $response) {
        try {
            $headers = $request->getServerParams();
            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $authToken = $config['authToken'];
            
            $req = $request->getParsedBody();
            
            $json = json_encode($req);
            $array = json_decode($json, TRUE);
            
            $array = $this->formatArray($array);
            
            $inputXml = $this->integrationService->buildXmlString($array);
            
            $fileName = date("c") . ".txt";

	        $content  = "[REQUEST] - " . date("c") . "\n" . $inputXml . "\n\n";
	        
	        $this->integrationService->saveRequestLog($fileName, $content, "RedemptionResults");
            
            if(!empty($headers['HTTP_AUTHORIZATION'])){
                
                $token = substr($headers['HTTP_AUTHORIZATION'], 7);
                
                if($token === $authToken){
                    $authToken = $this->getCurrentAuthToken();
                    
                    $requestHeader = [
                        "Content-Type: application/json",
                        "Authorization: Bearer $authToken"
                    ];
                    
                    $requestResponse = $this->sendRequest("/Redemption/" . $array['requestId'], 'GET', $requestHeader, []);
                    $error = false;
                    
                    if(empty($requestResponse)){
                        $responseXml = $this->integrationService->buildXmlString([], 'Não foi possivel comunicar com a api Madero Core Bancario.', true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na busca de resultados do resgate de saldo: Não foi possivel comunicar com a api Madero Core Bancario\n");
                    } 
                    
                    else if(!empty($requestResponse['isSuccess']) && $requestResponse['isSuccess'] == true){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, '', false);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Busca de resultados do resgate de saldo realizada com sucesso.\n");
                    }
                    
                    else if(!empty($requestResponse['errors']['message'])){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, $requestResponse['errors']['message'], true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na busca de resultados do resgate de saldo: " . $requestResponse['errors']['message'] . "\n");
                    }
                    
                    else{
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, "Ocorreu um erro inesperado.", true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na busca de resultados do resgate de saldo: Erro inesperado.\n");
                    }
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "RedemptionResults");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                } else{
                    $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao invalido', true);
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao invalido.\n");
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "RedemptionResults");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                }
            } else{
                $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao nao enviado', true);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao nao enviado.\n");
                
                $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                $this->integrationService->saveRequestLog($fileName, $content, "RedemptionResults");
                
                return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
            }
            
        } catch (\Exception $e) {
            $responseXml = $this->integrationService->buildXmlString([], 'Ocorreu um erro inesperado', true);
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro inesperado: " . $e->getMessage() . "\n");
            
            $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
            $this->integrationService->saveRequestLog($fileName, $content, "RedemptionResults");
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
        }
    }
    
    public function RedemptionConfirmation(Request $request, Response $response) {
        try {
            $headers = $request->getServerParams();
            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $authToken = $config['authToken'];
            
            $req = $request->getParsedBody();
            
            $json = json_encode($req);
            $array = json_decode($json, TRUE);
            
            $array = $this->formatArray($array);
            
            $inputXml = $this->integrationService->buildXmlString($array);
            
            $fileName = date("c") . ".txt";

	        $content  = "[REQUEST] - " . date("c") . "\n" . $inputXml . "\n\n";
	        
	        $this->integrationService->saveRequestLog($fileName, $content, "RedemptionConfirmation");
            
            if(!empty($headers['HTTP_AUTHORIZATION'])){
                
                $token = substr($headers['HTTP_AUTHORIZATION'], 7);
                
                if($token === $authToken){
                    $authToken = $this->getCurrentAuthToken();
                    
                    $requestHeader = [
                        "Content-Type: application/json",
                        "Authorization: Bearer $authToken"
                    ];
                    
                    $requestBody = $array;
                    
                    $requestResponse = $this->sendRequest("/Redemption/Confirm", 'POST', $requestHeader, $requestBody);
                    $error = false;
                    
                    if(empty($requestResponse)){
                        $responseXml = $this->integrationService->buildXmlString([], 'Não foi possivel comunicar com a api Madero Core Bancario.', true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na confirmação de resgate de saldo: Não foi possivel comunicar com a api Madero Core Bancario\n");
                    } 
                    
                    else if(!empty($requestResponse['isSuccess']) && $requestResponse['isSuccess'] == true){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, '', false);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Confirmação de resgate de saldo realizada com sucesso.\n");
                    }
                    
                    else if(!empty($requestResponse['errors']['message'])){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, $requestResponse['errors']['message'], true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na confirmação de resgate de saldo: " . $requestResponse['errors']['message'] . "\n");
                    }
                    
                    else{
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, "Ocorreu um erro inesperado.", true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na confirmação de resgate de saldo: Erro inesperado.\n");
                    }
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "RedemptionConfirmation");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                } else{
                    $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao invalido', true);
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao invalido.\n");
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "RedemptionConfirmation");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                }
            } else{
                $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao nao enviado', true);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao nao enviado.\n");
                
                $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                $this->integrationService->saveRequestLog($fileName, $content, "RedemptionConfirmation");
                
                return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
            }
            
        } catch (\Exception $e) {
            $responseXml = $this->integrationService->buildXmlString([], 'Ocorreu um erro inesperado', true);
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro inesperado: " . $e->getMessage() . "\n");
            
            $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
            $this->integrationService->saveRequestLog($fileName, $content, "RedemptionConfirmation");
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
        }
    }
    
    public function RedemptionConfirmationResults(Request $request, Response $response) {
        try {
            $headers = $request->getServerParams();
            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $authToken = $config['authToken'];
            
            $req = $request->getParsedBody();
            
            $json = json_encode($req);
            $array = json_decode($json, TRUE);
            
            $array = $this->formatArray($array);
            
            $inputXml = $this->integrationService->buildXmlString($array);
            
            $fileName = date("c") . ".txt";

	        $content  = "[REQUEST] - " . date("c") . "\n" . $inputXml . "\n\n";
	        
	        $this->integrationService->saveRequestLog($fileName, $content, "RedemptionConfirmationResults");
            
            if(!empty($headers['HTTP_AUTHORIZATION'])){
                
                $token = substr($headers['HTTP_AUTHORIZATION'], 7);
                
                if($token === $authToken){
                    $authToken = $this->getCurrentAuthToken();
                    
                    $requestHeader = [
                        "Content-Type: application/json",
                        "Authorization: Bearer $authToken"
                    ];
                    
                    $requestResponse = $this->sendRequest("/Redemption/Confirm/" . $array['requestId'], 'GET', $requestHeader, []);
                    $error = false;
                    
                    if(empty($requestResponse)){
                        $responseXml = $this->integrationService->buildXmlString([], 'Não foi possivel comunicar com a api Madero Core Bancario.', true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na busca de resultados da confirmação de resgate de saldo: Não foi possivel comunicar com a api Madero Core Bancario\n");
                    } 
                    
                    else if(!empty($requestResponse['isSuccess']) && $requestResponse['isSuccess'] == true){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, '', false);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Busca de resultados da confirmação de resgate de saldo realizada com sucesso.\n");
                    }
                    
                    else if(!empty($requestResponse['errors']['message'])){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, $requestResponse['errors']['message'], true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na busca de resultados da confirmação de resgate de saldo: " . $requestResponse['errors']['message'] . "\n");
                    }
                    
                    else{
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, "Ocorreu um erro inesperado.", true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na busca de resultados da confirmação de resgate de saldo: Erro inesperado.\n");
                    }
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "RedemptionConfirmationResults");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                } else{
                    $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao invalido', true);
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao invalido.\n");
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "RedemptionConfirmationResults");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                }
            } else{
                $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao nao enviado', true);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao nao enviado.\n");
                
                $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                $this->integrationService->saveRequestLog($fileName, $content, "RedemptionConfirmationResults");
                
                return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
            }
            
        } catch (\Exception $e) {
            $responseXml = $this->integrationService->buildXmlString([], 'Ocorreu um erro inesperado', true);
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro inesperado: " . $e->getMessage() . "\n");
            
            $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
            $this->integrationService->saveRequestLog($fileName, $content, "RedemptionConfirmationResults");
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
        }
    }
    
    public function Cancel(Request $request, Response $response) {
        try {
            $headers = $request->getServerParams();
            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $authToken = $config['authToken'];
            
            $req = $request->getParsedBody();
            
            $json = json_encode($req);
            $array = json_decode($json, TRUE);
            
            $array = $this->formatArray($array);
            
            $inputXml = $this->integrationService->buildXmlString($array);
            
            $fileName = date("c") . ".txt";

	        $content  = "[REQUEST] - " . date("c") . "\n" . $inputXml . "\n\n";
	        
	        $this->integrationService->saveRequestLog($fileName, $content, "Cancel");
            
            if(!empty($headers['HTTP_AUTHORIZATION'])){
                
                $token = substr($headers['HTTP_AUTHORIZATION'], 7);
                
                if($token === $authToken){
                    $authToken = $this->getCurrentAuthToken();
                    
                    $requestHeader = [
                        "Content-Type: application/json",
                        "Authorization: Bearer $authToken"
                    ];
                    
                    $requestBody = $array;
                    
                    $requestResponse = $this->sendRequest("/Redemption/Cancel", 'POST', $requestHeader, $requestBody);
                    $error = false;
                    
                    if(empty($requestResponse)){
                        $responseXml = $this->integrationService->buildXmlString([], 'Não foi possivel comunicar com a api Madero Core Bancario.', true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro no cancelamento de resgate de saldo: Não foi possivel comunicar com a api Madero Core Bancario\n");
                    } 
                    
                    else if(!empty($requestResponse['isSuccess']) && $requestResponse['isSuccess'] == true){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, '', false);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Cancelamento de resgate de saldo realizada com sucesso.\n");
                    }
                    
                    else if(!empty($requestResponse['errors']['message'])){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, $requestResponse['errors']['message'], true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro no cancelamento de resgate de saldo: " . $requestResponse['errors']['message'] . "\n");
                    }
                    
                    else{
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, "Ocorreu um erro inesperado.", true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro no cancelamento de resgate de saldo: Erro inesperado.\n");
                    }
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "Cancel");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                } else{
                    $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao invalido', true);
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao invalido.\n");
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "Cancel");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                }
            } else{
                $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao nao enviado', true);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao nao enviado.\n");
                
                $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                $this->integrationService->saveRequestLog($fileName, $content, "Cancel");
                
                return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
            }
            
        } catch (\Exception $e) {
            $responseXml = $this->integrationService->buildXmlString([], 'Ocorreu um erro inesperado', true);
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro inesperado: " . $e->getMessage() . "\n");
            
            $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
            $this->integrationService->saveRequestLog($fileName, $content, "Cancel");
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
        }
    }
    
    private function getCurrentAuthToken(){
        if(file_exists(__DIR__ . "/../../service/authToken.json")){
            $tokenFile = file_get_contents(__DIR__ . "/../../service/authToken.json");
            $tokenData = json_decode($tokenFile, true);
        } else{
            $tokenData = [
                "token" => '',
                "timestamp" => strtotime('01-01-1900 00:00:00')
            ];
        }
        
        $isTokenValid = false;
        
        $currentTime = strtotime(date('d-m-Y H:i:s'));
        
        if(!empty($tokenData)){
            if($currentTime - $tokenData['timestamp'] < 3000){
                $isTokenValid = true;
            }
        }
        
        if($isTokenValid){
            return $tokenData['token'];
        } else{
            return $this->renewAuthToken();
        }
    }
    
    public function renewAuthToken(){
        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Iniciando servico de renovacao de token." . PHP_EOL);
        
        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);
        
        $authUrl = !empty($config['authUrl']) ? $config['authUrl'] : null;
        $clientId = !empty($config['clientId']) ? $config['clientId'] : null;
        $apiScope = !empty($config['apiScope']) ? $config['apiScope'] : null;
        $apiGrantType = !empty($config['apiGrantType']) ? $config['apiGrantType'] : null;
        $clientSecret = !empty($config['clientSecret']) ? $config['clientSecret'] : null;
        $username = !empty($config['authUsername']) ? $config['authUsername'] : null;
        $password = !empty($config['authPassword']) ? $config['authPassword'] : null;
        
        //valid token check
        if(file_exists(__DIR__ . "/../../service/authToken.json")){
            $tokenFile = file_get_contents(__DIR__ . "/../../service/authToken.json");
            $tokenData = json_decode($tokenFile, true);
        } else{
            $tokenData = [
                "token" => '',
                "timestamp" => strtotime('01-01-1900 00:00:00')
            ];
        }
        
        $isTokenValid = false;
        
        $currentTime = strtotime(date('d-m-Y H:i:s'));
        
        if(!empty($tokenData)){
            if($currentTime - $tokenData['timestamp'] < 3000){
                $isTokenValid = true;
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." O token atual ainda e valido." . PHP_EOL);
                
                return $tokenData['token'];
            }
        }
        
        if(!$isTokenValid){
            $requestUrl = $authUrl;
            
            $requestHeader = [
                "Content-Type: application/x-www-form-urlencoded"
            ];
            $apiParams = [
                'client_id' => $clientId,
                'client_secret' => $clientSecret,
                'scope' => $apiScope,
                'grant_type' => $apiGrantType,
                'username' => $username,
                'password' => $password,
            ];
            $queryParams = http_build_query($apiParams);
            $curl = curl_init();
            
            curl_setopt_array($curl, [
                CURLOPT_URL             => $requestUrl,
                CURLOPT_RETURNTRANSFER  => true,
                CURLOPT_ENCODING        => '',
                CURLOPT_CONNECTTIMEOUT  => 5,
                CURLOPT_TIMEOUT         => 90,
                CURLOPT_FOLLOWLOCATION  => true,
                CURLOPT_SSL_VERIFYPEER  => false,
                CURLOPT_USERAGENT       => "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/20100101 Firefox/17.0",
                CURLOPT_HTTP_VERSION    => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST   => "POST",
                CURLOPT_POSTFIELDS      => $queryParams,
                CURLOPT_HTTPHEADER      => $requestHeader
            ]);
            
            $curlResponse = curl_exec($curl);
            $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    
            if (!empty(curl_error($curl)) || ($httpCode != 200) || (stripos($curlResponse, "<html>")) || (stripos($curlResponse, "Fatal error"))) {
                $fileName = "EXP-".date("H-i-s") . ".txt";
    
                $this->util->writeLog('network', $fileName, $this->util->formatNetworkError([
                    'eventTime' => date("d/m/Y H:i:s"),
                    'header'    => $requestHeader,
                    'url'       => $requestUrl,
                    'method'    => "POST",
                    'params'    => $queryParams,
                    'httpCode'  => $httpCode,
                    'error'     => curl_error($curl),
                    'response'  => $curlResponse
                ]));

                throw new \Exception("Nao foi possível gerar o token de autenticação na API do Madero.");
            } else{
                curl_close($curl);
                
                $return = json_decode($curlResponse, true);
                
                $newToken = [
                    "token" => $return['access_token'],
                    "timestamp" => strtotime(date('d-m-Y H:i:s'))
                ];
                
                $filePath = __DIR__ . "/../../service/authToken.json";
                file_put_contents($filePath, json_encode($newToken));

                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." O token foi renovado com sucesso." . PHP_EOL);
                
                return $return['access_token'];
            }
        }
    }
    
    private function sendRequest($route, $method, $requestHeader, $requestBody, $query = null){
        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);
        
        $url = !empty($config['baseURL']) ? $config['baseURL'] : null;
        
        $requestUrl = $url.$route;
        
        if(!empty($query)){
            $requestUrl = $requestUrl.'?'.$query;
        }
        
        $curl = curl_init();
        
        if(empty($requestBody)){
            curl_setopt_array($curl, [
                CURLOPT_URL             => $requestUrl,
                CURLOPT_RETURNTRANSFER  => true,
                CURLOPT_ENCODING        => '',
                CURLOPT_CONNECTTIMEOUT  => 5,
                CURLOPT_TIMEOUT         => 90,
                CURLOPT_FOLLOWLOCATION  => true,
                CURLOPT_SSL_VERIFYPEER  => false,
                CURLOPT_USERAGENT       => "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/20100101 Firefox/17.0",
                CURLOPT_HTTP_VERSION    => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST   => $method,
                CURLOPT_HTTPHEADER      => $requestHeader
            ]);
        } else{
            curl_setopt_array($curl, [
                CURLOPT_URL             => $requestUrl,
                CURLOPT_RETURNTRANSFER  => true,
                CURLOPT_ENCODING        => '',
                CURLOPT_CONNECTTIMEOUT  => 5,
                CURLOPT_TIMEOUT         => 90,
                CURLOPT_FOLLOWLOCATION  => true,
                CURLOPT_SSL_VERIFYPEER  => false,
                CURLOPT_USERAGENT       => "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/20100101 Firefox/17.0",
                CURLOPT_HTTP_VERSION    => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST   => $method,
                CURLOPT_POSTFIELDS      => json_encode($requestBody),
                CURLOPT_HTTPHEADER      => $requestHeader
            ]);
        }
        
        $curlResponse = curl_exec($curl);
        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        if (!empty(curl_error($curl)) || ($httpCode != 200) || (stripos($curlResponse, "<html>")) || (stripos($curlResponse, "Fatal error"))) {
            $fileName = "EXP-".date("H-i-s") . ".txt";

            $this->util->writeLog('network', $fileName, $this->util->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => $requestHeader,
                'url'       => $url.$route,
                'method'    => $method,
                'params'    => json_encode($requestBody),
                'httpCode'  => $httpCode,
                'error'     => curl_error($curl),
                'response'  => $curlResponse
            ]));
        }
        
        curl_close($curl);
        
        return json_decode($curlResponse, true);
    }

    private function writeEventsLog($events) {
        foreach($events as $event) {

            $reference = $event['correlationId'];
            $eventCode = $event['code'];

            $fileName = round(microtime(true)*1000) . "-$eventCode.json";
            $this->util->writeLog("events", $fileName, $event);
        }
    }
    
    public function welcome(Request $request, Response $response) {
        $sql        = "SELECT CDFILIAL FROM FILIAL";
        $result     = $this->connection->fetchAll($sql);

        $versionInfo = $this->getVersionInfo();

        return $response->withJson([
            'version'     => $versionInfo['version'],
            'revision'    => $versionInfo['revision'],
            'date'        => $versionInfo['date'],
            'branchs'     => count($result)
        ]);
    }

    private function getVersionInfo() {
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }

    private function getClientName() {
        global $CLIENT_NAME;
        return $CLIENT_NAME;
    }
    
    private function formatArray($array) {
        $formattedArray = [];
        
        foreach($array as $arrayElement){
            $indexName = array_search($arrayElement, $array);
            
            if(is_array($arrayElement)){
                if($indexName === 'data'){
                    if((sizeof($arrayElement) > 1) && (!empty($arrayElement[0]))){
                        foreach($arrayElement as $arrElemChild){
                            if(is_array($arrElemChild)){
                                array_push($formattedArray, $this->formatArray($arrElemChild));
                            } else{
                                array_push($formattedArray, $arrElemChild);
                            }
                        }
                    } else{
                        array_push($formattedArray, $this->formatArray($arrayElement));
                    }
                } else{
                    $formattedArray[$indexName] = $this->formatArray($arrayElement);
                }
            } else{
                if($indexName === 'data'){
                    array_push($formattedArray, $arrayElement);
                } else{
                    $formattedArray[$indexName] = $arrayElement;
                }
            }
            
            unset($array[$indexName]);
        }
        
        return $formattedArray;
    }
}
